package com.jpattern.orm;

import java.io.File;
import java.io.FileInputStream;
import java.math.BigDecimal;

import com.jpattern.logger.ILogger;
import com.jpattern.orm.JPOrm;
import com.jpattern.orm.connection.ConnectionWrapper;
import com.jpattern.orm.dialect.IDialect;
import com.jpattern.orm.logger.OrmLogger;
import com.jpattern.orm.script.IScriptExecutor;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.session.ISessionProvider;
import com.jpattern.orm.transaction.ITransaction;

import junit.framework.TestCase;

/**
 * 
 * @author Francesco Cina
 *
 * 20/mag/2011
 */
public abstract class BaseTest extends TestCase {

	private Chronometer chronometer = new Chronometer();
	private ILogger logger;
	private static ISessionProvider sessionProvider;
	private static IDialect dialect;

    public BaseTest() {
    }

    public BaseTest(String name) {
        super(name);
    }
    
    protected void setUp() throws Exception {
        super.setUp();
        
        chronometer.avanzaDaCapo();
        
        logger = OrmLogger.getOrmLogger(getClass());
        
        logger.info("setUp","===================================================================");
        logger.info("setUp","BEGIN TEST " + getName());
        logger.info("setUp","OPEN_CONNECTIONS: " + ConnectionWrapper.OPEN_CONNECTIONS );
        logger.info("setUp","CLOSED_CONNECTIONS: " + ConnectionWrapper.CLOSED_CONNECTIONS );
        logger.info("setUp","TOTAL_CONNECTIONS: " + ConnectionWrapper.TOTAL_CONNECTIONS );
        logger.info("setUp","COMMITTED_CONNECTIONS: " + ConnectionWrapper.COMMITTED_CONNECTIONS );
        logger.info("setUp","ROLLBACKED_CONNECTIONS: " + ConnectionWrapper.ROLLBACKED_CONNECTIONS );
        logger.info("setUp","===================================================================");

    }


	protected void tearDown() throws Exception {
        super.tearDown();
        
        chronometer.ferma();
        
        String time = new BigDecimal(chronometer.leggi() ).divide(new BigDecimal(1000)).toString();
        
        logger.info("setUp","===================================================================");
        logger.info("setUp","END TEST " + getName());
        logger.info("setUp","OPEN_CONNECTIONS: " + ConnectionWrapper.OPEN_CONNECTIONS );
        logger.info("setUp","CLOSED_CONNECTIONS: " + ConnectionWrapper.CLOSED_CONNECTIONS );
        logger.info("setUp","TOTAL_CONNECTIONS: " + ConnectionWrapper.TOTAL_CONNECTIONS );
        logger.info("setUp","COMMITTED_CONNECTIONS: " + ConnectionWrapper.COMMITTED_CONNECTIONS );
        logger.info("setUp","ROLLBACKED_CONNECTIONS: " + ConnectionWrapper.ROLLBACKED_CONNECTIONS );
        logger.info("setUp","Execution time: " + time + " seconds");
        logger.info("setUp","===================================================================");
    }
	
	protected String getTestInputBasePath() throws Exception {
		return IJpOrmTestsConstants.TEST_FILE_INPUT_BASE_PATH;
	}
	
	protected String getTestOutputBasePath() throws Exception {
		mkDir(IJpOrmTestsConstants.TEST_FILE_OUTPUT_BASE_PATH);
		return IJpOrmTestsConstants.TEST_FILE_OUTPUT_BASE_PATH;
	}
	
	protected void mkDir( String dirPath ) throws Exception {
		File path = new File(dirPath);
		if (!path.exists()) {
			path.mkdirs();
		}
	}
	
	public static void init(ISessionProvider sessionProvider, String createScriptFileName, String dropScriptFileName, IDialect dialect) throws Exception {
		BaseTest.sessionProvider = sessionProvider;
		BaseTest.dialect = dialect;
		assertNotNull(sessionProvider);
		IJPOrm jpOrm = new JPOrm("",sessionProvider, dialect);
		ISession session = jpOrm.session();
		ITransaction tx = session.getTransaction();
		IScriptExecutor scriptExecutor = session.scriptExecutor();
		try {
			scriptExecutor.execute(new FileInputStream(dropScriptFileName));
		} catch (Exception e) {}
		scriptExecutor.execute(new FileInputStream(createScriptFileName));
		tx.commit();
	}
	
	protected IJPOrm getJPOrm() {
		return new JPOrm("",sessionProvider, dialect);
	}

	public ISessionProvider getSessionProvider() {
		return sessionProvider;
	}
}
	
