package com.jpattern.orm;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.jpattern.orm.dialect.DefaultDialect;
import com.jpattern.orm.test.AutoIdTest;
import com.jpattern.orm.test.EmployeeTest;
import com.jpattern.orm.test.PeopleMultipleTest;
import com.jpattern.orm.test.PeopleTest;
import com.jpattern.orm.test.h2.BlobClob_ByteArray_Test;
import com.jpattern.orm.test.h2.BlobClob_InputStream_Reader_Test;
import com.jpattern.orm.test.h2.BlobClob_String_Test;
import com.jpattern.orm.test.h2.People2Test;
import com.jpattern.orm.test.h2.ZooPeopleTest;
import com.jpattern.orm.test.mapper.ClassMapperGeneratorTest;
import com.jpattern.orm.test.query.PlainSqlExecutorsTest;
import com.jpattern.orm.test.query.QueryExecutionTest;
import com.jpattern.orm.test.query.forupdate.QuerySelectForUpdateExecutionTest;
import com.jpattern.orm.test.script.ScriptExecutorTest;
import com.jpattern.orm.test.session.SqlExecutorsTest;
import com.jpattern.orm.test.transaction.EmployeeTransactionTest;
import com.jpattern.orm.test.transaction.JdbcTemplatePeopleTest;

/**
 * 
 * @author Francesco Cina
 * 
 *         20/mag/2011
 */
public class ComplianceH2Tests extends TestCase {
	/**
	 * Create the test case
	 * 
	 * @param testName
	 *            name of the test case
	 */
	public ComplianceH2Tests(String testName) {
		super(testName);
	}

	/**
	 * @return the suite of tests being tested
	 */
	public static Test suite() {
		
		try {
			BaseTest.init(SessionProviderProxy.getSessionsProvider().getH2SessionProvider(), 
					IJpOrmTestsConstants.H2_CREATE_SCRIPT, 
					IJpOrmTestsConstants.H2_DROP_SCRIPT,
					new DefaultDialect());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		
		TestSuite suite = new TestSuite("COMPLIANCE H2 TEST SUITE FOR jPattern ORM");

		suite.addTestSuite(EmployeeTest.class);
		suite.addTestSuite(ClassMapperGeneratorTest.class);
		suite.addTestSuite(PeopleTest.class);
		suite.addTestSuite(EmployeeTransactionTest.class);
		suite.addTestSuite(QueryExecutionTest.class);
		suite.addTestSuite(PeopleMultipleTest.class);
		suite.addTestSuite(ScriptExecutorTest.class);
		suite.addTestSuite(AutoIdTest.class);
		suite.addTestSuite(QuerySelectForUpdateExecutionTest.class);
		suite.addTestSuite(JdbcTemplatePeopleTest.class);
		
		//H2 specific tests
		suite.addTestSuite(People2Test.class);
		suite.addTestSuite(BlobClob_InputStream_Reader_Test.class);
		suite.addTestSuite(BlobClob_String_Test.class);
		suite.addTestSuite(BlobClob_ByteArray_Test.class);
		suite.addTestSuite(ZooPeopleTest.class);
		
		//test with Oracle SEQUENCE notation
		suite.addTestSuite(SqlExecutorsTest.class);
		suite.addTestSuite(PlainSqlExecutorsTest.class);
		
		return suite;
	}
}
