package com.jpattern.orm.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.query.IOrmQuery;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.transaction.ITransaction;
import com.jpattern.orm.test.domain.People;

/**
 * 
 * @author Francesco Cina
 *
 * 20/mag/2011
 */
public class PeopleMultipleTest extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testCrudPeople() throws Exception {
		IJPOrm jpOrm =getJPOrm();
		
		jpOrm.register(People.class);
		
        List<People> peoples = new ArrayList<People>();
        peoples.add(createPeople("1"));
        peoples.add(createPeople("2"));
        peoples.add(createPeople("3"));
        peoples.add(createPeople("4"));
		
		// CREATE
        ISession conn = jpOrm.session();
        ITransaction tx = conn.getTransaction();
        conn.save(peoples);
        tx.commit();
        
        for (People people : peoples) {
        	System.out.println("People [" + people.getFirstname() + "]saved with id: " + people.getId());
        }
        
        // LOAD
        IOrmQuery<People> peopleQuery1 = conn.findQuery(People.class);
        assertNotNull(peopleQuery1);
        Collection<Long> values = new ArrayList<Long>();
        for ( People people : peoples) {
        	values.add( people.getId() );
        }
		peopleQuery1.where().in("People.id", values );
        List<People> peopleLoad1 = peopleQuery1.findList();
        
        compare(peoples, peopleLoad1);
        
        //UPDATE
        tx = conn.getTransaction();
        for ( People people : peoples) {
        	people.setFirstname( people.getFirstname() + "-updated-" + new Date().getTime() ) ;
        }
        conn.update(peoples);
        tx.commit();
        
        // LOAD
        IOrmQuery<People> peopleQuery2 = conn.findQuery(People.class);
        assertNotNull(peopleQuery2);
		peopleQuery2.where().in("People.id", values );
        List<People> peopleLoad2 = peopleQuery2.findList();
        
        compare(peoples, peopleLoad2);
        
        
        //DELETE
        tx = conn.getTransaction();
        conn.delete(peopleLoad2);
        tx.commit();
        
        //LOAD
        IOrmQuery<People> peopleQuery3 = conn.findQuery(People.class);
        assertNotNull(peopleQuery3);
		peopleQuery3.where().in("People.id", values );
        List<People> peopleLoad3 = peopleQuery3.findList();
        assertTrue( peopleLoad3.isEmpty() );
        
	}
	
	private void compare(List<People> peoples, List<People> peoplesLoad) {
		assertEquals(peoples.size(), peoplesLoad.size());
		for ( People people : peoples) {
			System.out.println("check people: " + people.getId());
			boolean found = false;
			for (People peopleLoad : peoplesLoad) {
				if (peopleLoad.getId() == people.getId()) {
					found = true;
					assertEquals(people.getFirstname(), peopleLoad.getFirstname());
					assertEquals(people.getLastname(), peopleLoad.getLastname());
				}
			}
			assertTrue(found);
		}
	}

	private People createPeople(String name) {
        long id = new Date().getTime();
		People people = new People();
		people.setId( id );
		people.setFirstname( name );
		people.setLastname( "lastname of " + name );
		return people;
	}

}
