package com.jpattern.orm.test.h2;

import java.io.InputStream;
import java.io.Reader;
import java.util.Date;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.test.domain.Zoo_People;
import com.jpattern.orm.util.OrmUtil;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.transaction.ITransaction;

/**
 * 
 * @author Francesco Cina
 *
 * 20/mag/2011
 */
public class ZooPeopleTest extends BaseTest {

	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testCrudZoo() throws Exception {
		IJPOrm jpOrm =getJPOrm();
		
		jpOrm.register(Zoo_People.class);
		
        long id = new Date().getTime();
        
        String text1 = "BINARY STRING TEST 1 " + id;
		InputStream is = OrmUtil.convertStringToStream(text1, OrmUtil.UTF8);

        String text2 = "BINARY STRING TEST 2 " + id;
		Reader reader = OrmUtil.convertStringToReader(text2);
		
		Zoo_People Zoo_People = new Zoo_People();
		Zoo_People.setId( id );
		Zoo_People.setFirstname( "Zoo_People" );
		Zoo_People.setLastname("Wizard");
		Zoo_People.setFirstblob(is);
		Zoo_People.setFirstclob(reader);
		
		// CREATE
        ISession conn = jpOrm.session();
        ITransaction tx = conn.getTransaction();
        conn.save(Zoo_People);
        tx.commit();
        
        reader.close();
        is.close();
        
        System.out.println("Zoo_People saved with id: " + Zoo_People.getId());
        assertFalse( id == Zoo_People.getId() );
        id = Zoo_People.getId();
        
        // LOAD
        tx = conn.getTransaction();
        Zoo_People peopleLoad1 = conn.find(Zoo_People.class, new Object[]{id});
        assertNotNull(peopleLoad1);
        assertEquals( Zoo_People.getId(), peopleLoad1.getId() );
        assertEquals( Zoo_People.getFirstname(), peopleLoad1.getFirstname() );
        assertEquals( Zoo_People.getLastname(), peopleLoad1.getLastname() );
        
        String retrieved1 = OrmUtil.convertStreamToString( peopleLoad1.getFirstblob(), OrmUtil.UTF8, false);
        System.out.println("Retrieved1 String " + retrieved1);
        assertEquals( text1 , retrieved1 );
        
        String retrieved2 = OrmUtil.convertReaderToString( peopleLoad1.getFirstclob() , false);
        System.out.println("Retrieved2 String " + retrieved2);
        assertEquals( text2 , retrieved2 );
        
        
        //UPDATE
        peopleLoad1.setFirstname("Wizard name");
        conn.update(peopleLoad1);
        tx.commit();
        
        // LOAD
        tx = conn.getTransaction();
        Zoo_People peopleLoad2 = conn.find(Zoo_People.class, new Object[]{id});
        assertNotNull(peopleLoad2);
        assertEquals( peopleLoad1.getId(), peopleLoad2.getId() );
        assertEquals( peopleLoad1.getFirstname(), peopleLoad2.getFirstname() );
        assertEquals( peopleLoad1.getLastname(), peopleLoad2.getLastname() );
        
        //DELETE
        conn.delete(peopleLoad2);
        Zoo_People peopleLoad3 = conn.find(Zoo_People.class, new Object[]{id});
        assertNull(peopleLoad3);
        tx.commit();
        
	}

}
