package com.jpattern.orm.test.query.forupdate;

import java.util.Date;
import java.util.Random;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.query.IOrmQuery;
import com.jpattern.orm.query.IOrmRowMapper;
import com.jpattern.orm.query.LockMode;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.transaction.ITransaction;
import com.jpattern.orm.test.domain.Employee;
import com.jpattern.orm.test.domain.Zoo_People;

/**
 * 
 * @author Francesco Cina'
 *
 * 30/ago/2011
 */
public class QuerySelectForUpdateExecutionTest extends BaseTest {

	private long THREAD_SLEEP = 250l;
	
	protected void setUp() throws Exception {
		super.setUp();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	
	public void testQuery1() throws Exception {
		IJPOrm jpOrm =getJPOrm();
		jpOrm.register(Employee.class);
		jpOrm.register(Zoo_People.class);

		ISession session =  jpOrm.session();
		Employee employeeLocked = createEmployee(jpOrm);
		Employee employeeUnlocked = createEmployee(jpOrm);
		
		ActorLockForUpdate actor1 = new ActorLockForUpdate(jpOrm, employeeLocked.getId(), LockMode.FOR_UPDATE, "locked");
		Thread thread1 = new Thread( actor1 );
		thread1.start();
		
		Thread.sleep(THREAD_SLEEP / 5);
		
		ActorLockForUpdate actor2 = new ActorLockForUpdate(jpOrm, employeeLocked.getId(), LockMode.FOR_UPDATE, "locked2");
		Thread thread2 = new Thread( actor2 );
		thread2.start();
		
		thread1.join();
		thread2.join();
		assertFalse(actor1.exception);
		assertFalse(actor2.exception);
		
		assertEquals( "name_locked_locked2" ,  session.find(Employee.class, employeeLocked.getId()).getName() );
		
		deleteEmployee(jpOrm, employeeLocked);
		deleteEmployee(jpOrm, employeeUnlocked);
	}
	
	
	
	
	public class ActorLockForUpdate implements Runnable {

		private final IJPOrm jpOrm;
		private final LockMode lockMode;
		private final String name;
		private final long employeeId;
		boolean exception = false;
		
		public ActorLockForUpdate(IJPOrm jpOrm, long employeeId, LockMode lockMode, String name) {
			this.jpOrm = jpOrm;
			this.employeeId = employeeId;
			this.lockMode = lockMode;
			this.name = name;
		}
		
		@Override
		public void run() {
			System.out.println("Run: " + name);
			final ISession session;
			try {
				session = jpOrm.session();
				
				ITransaction tx = session.getTransaction();
				
				IOrmQuery<Employee> query = session.findQuery(Employee.class);
				query.where().eq("Employee.id", employeeId);
				query.setLockMode(lockMode);
				System.out.println("Thread " + name + " executing query [" + query.renderSql() + "]");
				
				IOrmRowMapper<Employee> srr = new IOrmRowMapper<Employee>() {
					@Override
					public void read(Employee employee, int rowCount) {
						System.out.println("Thread " + name + " - employee.getName() = [" + employee.getName() + "]");
						assertNotNull(employee);
						
						try {
							Thread.sleep(THREAD_SLEEP);
						} catch (InterruptedException e) {
						}
						
						employee.setName( employee.getName() + "_" + name);
						System.out.println("Thread " + name + " updating employee");
						session.update(employee);						
					}
				};
				query.find(srr);
				
				tx.commit();
				

			} catch (Exception e) {
				e.printStackTrace();
				exception = true;
			}
		}

	}
	
	
	
	private Employee createEmployee(IJPOrm jpOrm) {
		ISession ormSession = jpOrm.session();
		ITransaction tx = ormSession.getTransaction();
        long id = new Date().getTime() + new Random().nextInt();
		Employee employee = new Employee();
		employee.setId( id );
		employee.setAge( 44 );
		employee.setEmployeeNumber( ("" + id).substring(0, 9) );
		employee.setName("name");
		employee.setSurname("Cina");
		ormSession.save(employee);
		tx.commit();
		return employee;
	}
	
	private void deleteEmployee(IJPOrm jpOrm, Employee employee) {
		ISession ormSession = jpOrm.session();
		ITransaction tx = ormSession.getTransaction();
		ormSession.delete(employee);
		tx.commit();
	}
	
	
	
}
