package com.jpattern.orm.test.script;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import com.jpattern.orm.BaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.JPOrm;
import com.jpattern.orm.query.IOrmQuery;
import com.jpattern.orm.script.IScriptExecutor;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.session.ISessionProvider;
import com.jpattern.orm.transaction.ITransaction;

/**
 * 
 * @author Francesco Cina
 *
 * 02/lug/2011
 */
public class ScriptExecutorTest extends BaseTest {

	private String filename;
	private IJPOrm jpOrm;

	protected void setUp() throws Exception {
		super.setUp();
		
		filename = getTestInputBasePath() + "/testReader1.sql";
		assertTrue( new File(filename).exists() );
	}

	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testScript() throws Exception {
		ISessionProvider sessionProvider = getSessionProvider();
		executeScript( sessionProvider );
		verifyData( sessionProvider );
	}

	private void executeScript(ISessionProvider sessionProvider) throws Exception {
		jpOrm = new JPOrm("",sessionProvider);
		
		ISession session = jpOrm.session();
		ITransaction tx = session.getTransaction();
		IScriptExecutor scriptExecutor = session.scriptExecutor();
		
		InputStream scriptStream = new FileInputStream(filename);
		scriptExecutor.execute(scriptStream);
		
		tx.commit();
		
	}
	
	private void verifyData(ISessionProvider sessionProvider) throws Exception {
		jpOrm = new JPOrm("",sessionProvider);
		jpOrm.register(TempTable.class);
		
		ISession session = jpOrm.session();
		 IOrmQuery<TempTable> query = session.findQuery(TempTable.class);
		 query.orderBy().asc("TempTable.id");
		 List<TempTable> result = query.findList();
		
		 System.out.println("result.size() = " + result.size());
		assertTrue( result.size() == 11 );
		
		List<String> expectedResult = new ArrayList<String>();
		expectedResult.add("one");
		expectedResult.add("two");
		expectedResult.add("three");
		expectedResult.add("four;");
		expectedResult.add("f'ive;");
		expectedResult.add("s'ix;");
		expectedResult.add("seven';{--ix;");
		expectedResult.add("height';{--ix;");
		expectedResult.add("ni';ne';{--ix;");
		expectedResult.add("ten';{--ix;");
		expectedResult.add("e'le;{--ven;");
		
		for ( int i=0 ; i<result.size() ; i++) {
			TempTable temp = result.get(i);
			System.out.println("check element id: " + temp.getId() + " - name: " + temp.getName());
			assertEquals( expectedResult.get(i) , temp.getName());
		}
		
	}
	
}
